@@ -108,7 +108,7 @@ FORMAT_REC fecommon_silc_formats[] = {
 	{ "notice_private_signed", "{notice {flag_signed}$0{pvtnotice_host $1}}$2", 3, { 0, 0, 0 } },
 	{ "notice_private_unknown", "{notice {flag_unknown}$0{pvtnotice_host $1}}$2", 3, { 0, 0, 0 } },
 	{ "notice_private_failed", "{notice {flag_failed}$0{pvtnotice_host $1}}$2", 3, { 0, 0, 0 } },
-	{ "many", "There are now $0 {channick_hilight $1}'s on this channel", 2, { 0, 0 } },
+	{ "many", "There are now $0 {channick_hilight $1}s on this channel", 2, { 0, 0 } },
 	{ "user_appears", "{channick_hilight $0} will appear as {channick_hilight $1}", 2, { 0, 0 } },
 
 
