@@ -2,15 +2,14 @@
 
   none.c
 
-  Author: Pekka Riikonen <priikone@poseidon.pspt.fi>
+  Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 1997 - 2000 Pekka Riikonen
+  Copyright (C) 1997 - 2007 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-  
+  the Free Software Foundation; version 2 of the License.
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -18,10 +17,10 @@
 
 */
 
-#include "silcincludes.h"
+#include "silc.h"
 #include "none.h"
 
-/* 
+/*
  * SILC Crypto API for None cipher (ie. no cipher) :)
  */
 
@@ -30,9 +29,9 @@ SILC_CIPHER_API_SET_KEY(none)
   return TRUE;
 }
 
-SILC_CIPHER_API_SET_KEY_WITH_STRING(none)
+SILC_CIPHER_API_SET_IV(none)
 {
-  return TRUE;
+
 }
 
 SILC_CIPHER_API_CONTEXT_LEN(none)
@@ -40,14 +39,14 @@ SILC_CIPHER_API_CONTEXT_LEN(none)
   return 1;
 }
 
-SILC_CIPHER_API_ENCRYPT_CBC(none)
+SILC_CIPHER_API_ENCRYPT(none)
 {
-  memcpy(dst, src, len);
+  memmove(dst, src, len);
   return TRUE;
 }
 
-SILC_CIPHER_API_DECRYPT_CBC(none)
+SILC_CIPHER_API_DECRYPT(none)
 {
-  memcpy(dst, src, len);
+  memmove(dst, src, len);
   return TRUE;
 }
